# Generated by Django 5.0.4 on 2024-07-09 14:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0023_remove_container_cpus_remove_project_max_containers_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='container',
            name='username',
            field=models.CharField(default='root', help_text='Username for the RStudio Rocker container.', max_length=255),
        ),
    ]
