# Generated by Django 5.0.4 on 2024-07-02 14:19

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0021_remove_container_tailscale_funnel_url_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='container',
            name='image_host',
            field=models.CharField(default='docker.io', help_text='The name of the host where the image should be pulled from', max_length=255),
        ),
        migrations.AlterField(
            model_name='project',
            name='git_repo_url',
            field=models.CharField(blank=True, help_text='URL of the Git repository. You can use SSH or HTTPS URLs. For example, you could use forgejo@code.osi.uni-mannheim.de:r-shepard/<repo_name>.git', max_length=255, null=True, validators=[django.core.validators.RegexValidator('^(forgejo@|git@|http:\\/\\/|https:\\/\\/).*(\\.git)$', 'Enter a valid Git URL.')], verbose_name='Git Repository URL'),
        ),
    ]
