# Generated by Django 5.0.4 on 2024-07-09 09:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0022_container_image_host_alter_project_git_repo_url'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='container',
            name='cpus',
        ),
        migrations.RemoveField(
            model_name='project',
            name='max_containers',
        ),
        migrations.RemoveField(
            model_name='project',
            name='max_cpus',
        ),
        migrations.RemoveField(
            model_name='project',
            name='max_ram',
        ),
    ]
