<div id="messages" hx-swap-oob="true">
    {% if messages %}
        <ul class="messages">
            {% for message in messages %}
                <li {% if message.tags %}class="{{ message.tags|lower }}"{% endif %}>{{ message }}</li>
            {% endfor %}
        </ul>
    {% endif %}
</div>
